#define G_LOG_DOMAIN "boilerplate-preferences"

#include "config.h"

#include "boilerplate-preferences.h"
#include "boilerplate-preferences-view.h"
#include "boilerplate-preferences-page.h"
#include "boilerplate-window.h"
#include "boilerplate-application.h"

#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:boilerplate-preferences
 * @title: BoilerplatePreferences
 * @short_description: A #GtkWindow instance created for #BoilerplateApplication
 * @include: boilerplate-preferences.h
 * @see_also: #GtkWindow
 *
 * The #BoilerplatePreferences is an instance #GtkWindow, and acts as
 * the settings window for the current #GtkApplication.
 **/

typedef struct _BoilerplatePreferencesPrivate
{
    BoilerplateWindow *window;
    GSettings *settings;

    GtkWidget *toolbar;
    GtkWidget *headerbar;
    GtkWidget *content;

    BoilerplatePreferencesView *view;
} BoilerplatePreferencesPrivate;

struct _BoilerplatePreferences
{
    GtkWindow parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplatePreferences, boilerplate_preferences, GTK_TYPE_WINDOW)

/*
 * _on_key_press:
 * @widget: The original #GtkWidget that created the #G_CALLBACK
 * @event: The #GdkEventKey or the key that was pressed to start the callback
 * @user_data: User data set when the signal handler was connected, in this
 * instance it should be a #GtkWidget (#BoilerplatePreferences).
 *
 * On a keypress from the user, check for a case in the switch that matchs the
 * keypress. If the keypress exists, run that code and return %TRUE. Otherwise
 * return %FALSE.
 *
 * Returns: #gboolean; %TRUE key press case found and code executed or %FALSE
 * no case found.
 */
static gboolean
_on_key_press (GtkWidget   *widget,
               GdkEventKey *event,
               gpointer     user_data)
{
    switch (event->keyval)
    {
        case GDK_KEY_Escape:
            gtk_widget_destroy (widget);
            return TRUE;

            break;

        case GDK_KEY_Q:
        case GDK_KEY_q:
            if (event->state & GDK_CONTROL_MASK)
            {
                gtk_widget_destroy (widget);
                return TRUE;
            }
            break;

        default:
            return FALSE;
    }

    return FALSE;
}

/*
 * _setup_page_general:
 * @object: An instance of #BoilerplatePreferences
 *
 * Setups general page.
 */
void
_setup_page_general (BoilerplatePreferences *self)
{
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (self);

    boilerplate_preferences_view_add_page (priv->view,
                                           "general",
                                           "General");

    GtkBuilder *builder = gtk_builder_new_from_resource (PACKAGE_RESOURCES "/ui/preferences/pages/general.ui");

    GtkWidget *page = GTK_WIDGET (gtk_builder_get_object (builder, "page-general"));

    boilerplate_preferences_view_add_child_to_page (priv->view,
                                                    "general",
                                                    GTK_WIDGET (page));

    g_settings_bind (priv->settings, "example-button",
                     GTK_WIDGET (gtk_builder_get_object (builder, "example_button")), "active",
                     G_SETTINGS_BIND_DEFAULT);

    g_settings_bind(priv->settings, "example-combo",
                    GTK_WIDGET (gtk_builder_get_object (builder, "example_combo")), "active-id",
                    G_SETTINGS_BIND_DEFAULT);
}

/*
 * _setup_page_plugins:
 * @object: An instance of #BoilerplatePreferences
 *
 * Setups plugins page.
 */
void
_setup_page_plugins (BoilerplatePreferences *self)
{
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (self);

    boilerplate_preferences_view_add_page (priv->view,
                                           "plugins",
                                           "Plugins");
}

/*
 * boilerplate_preferences_add_page:
 * @object: An instance of #BoilerplatePreferences
 *
 * Setups all pages.
 */
void
_setup_pages (BoilerplatePreferences *self)
{
    _setup_page_general (self);
    _setup_page_plugins (self);
}

/*
 * boilerplate_window_constructed:
 * @object: An instance of #BoilerplateWindow
 *
 * The constructed virtual function sets first-run settings for the
 * #BoilerplateWindow object.
 *
 * It sets:
 *  *
 */
static void
boilerplate_preferences_constructed (GObject *object)
{
    BoilerplatePreferences *self = BOILERPLATE_PREFERENCES (object);
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (self);

    priv->view = boilerplate_preferences_view_new ();

    gtk_container_add (GTK_CONTAINER (priv->content),
                       GTK_WIDGET (priv->view));

    /* Setup Defaults */
    _setup_pages (self);

    G_OBJECT_CLASS (boilerplate_preferences_parent_class)->constructed (object);
}

/*
 * boilerplate_preferences_finalize:
 * @object: An instance of #BoilerplatePreferences
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
boilerplate_preferences_finalize (GObject *object)
{
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (BOILERPLATE_PREFERENCES (object));

    g_clear_object (&priv->settings);

    G_OBJECT_CLASS (boilerplate_preferences_parent_class)->finalize (object);
}

/*
 * boilerplate_preferences_init:
 * @self: An instance of #BoilerplatePreferences
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
boilerplate_preferences_init (BoilerplatePreferences *self)
{
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (self);

    gtk_widget_init_template (GTK_WIDGET (self));

     /* Set window title */
    gtk_window_set_title (GTK_WINDOW (self), _("Preferences"));

    /* Set default window decorations (a.k.a GtkHeaderBar) */
    gtk_window_set_titlebar (GTK_WINDOW (self), priv->headerbar);
    gtk_container_add (GTK_CONTAINER (priv->headerbar), priv->toolbar);

    /* Connect escape key */
    g_signal_connect (G_OBJECT (self),
                      "key_press_event",
                      G_CALLBACK (_on_key_press),
                      self);

    /* Bind settings from GUI */
    priv->settings = g_settings_new (PACKAGE_ID);
}

/*
 * boilerplate_preferences_class_init:
 * @klass: An instance of #BoilerplatePreferencesClass
 *
 * Initializes the various virtural functions in #BoilerplatePreferencesClass.
 * In addtition it connects the various #GtkWidgets from the gresource file
 * using gtk_widget_class_set_template_from_resource().
 */
static void
boilerplate_preferences_class_init (BoilerplatePreferencesClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->constructed = boilerplate_preferences_constructed;
    object_class->finalize = boilerplate_preferences_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/preferences.ui");

    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, toolbar);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, headerbar);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, content);
}

/**
 * boilerplate_preferences_new:
 * @window: A #GtkApplicationWindow instance
 *
 * Creates a new #BoilerplatePreferences, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * BoilerplatePreferences *preferences;
 *
 * preferences = boilerplate_preferences_new (window);
 * ]|
 *
 * When creating a boilerplate_preferences_new() a #BoilerplateWindow will need
 * to be passed. This is because #BoilerplatePreferences acts as a modal window
 * to the master window #BoilerplateWindow attached to the
 * #BoilerplateApplication.
 *
 * Returns: #BoilerplatePreferences; a sub-object of #GtkWindow
 */
BoilerplatePreferences *
boilerplate_preferences_new (BoilerplateWindow *window)
{
    return g_object_new (BOILERPLATE_TYPE_PREFERENCES,
                         "transient-for", window,
                         NULL);
}
